class PROJECT:
    # PROJECT ----------------------------------------------
    NAME_IMPORT: str = "pyqt_templates"
    KEYWORDS: list[str] = [
        "pyqt",
        "pyqt templates", "pyqt guide",
        "pyqt usage", "pyqt examples", "pyqt usage examples", "pyqt help", "pyqt help examples",
        "pyqt signals",
        "pyqt highlighter",
    ]
    # README --------------------------------------------------
    # add DOUBLE SPACE at the end of all lines! for correct representation in MD-viewers
    DESCRIPTION_SHORT: str = "pyqt help examples and some other useful aux_types (overloaded pyqt classes)"
    DESCRIPTION_LONG: str = """Designed for ..."""
    FEATURES: list[str] = [
        # "feat1",
        # ["feat2", "block1", "block2"],

        "good template for TableView/Model/Signals",
    ]

    # HISTORY -----------------------------------------------
    VERSION: tuple[int, int, int] = (0, 1, 6)
    TODO: list[str] = [
        "add Events for TM/TV/PTE/...",
    ]
    FIXME: list[str] = [
        "..."
    ]
    NEWS: list[str] = [
        "[annotAttrs]apply new ver",
    ]


# =====================================================================================================================
